/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm.type;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.util.swing.ASPanel;
import org.openide.WizardDescriptor;
import org.openide.util.Exceptions;

/**
 *
 * @author monezz
 */
public class FarmDetailsPanel extends ASPanel implements PropertyChangeListener {

    private Farm farm = new Farm();
    // indicating change from the panel to the farm
    private boolean outboundChange = false;
    // indicating change from the farm to the panel
    private boolean inboundChange = false;

    /**
     * Creates new form FarmWizardDetailsVisual
     */
    public FarmDetailsPanel(Farm farm) {
        this.farm = farm;
        initComponents();
        setOpaque(false);
        fieldName.setText(farm.getName());
        fieldRegistrationNumber.setText(farm.getRegistrationNumber());
        fieldName.getDocument().addDocumentListener(new NameChangeListener());
        fieldRegistrationNumber.getDocument().addDocumentListener(new RegistrationChangeListener());
        farm.addPropertyChangeListener(this);

    }

    public Farm getFarm() {
        return farm;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        bindingGroup = new org.jdesktop.beansbinding.BindingGroup();

        icon = new javax.swing.JLabel();
        nameLabel = new javax.swing.JLabel();
        registrationNumberLabel = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        fieldRegistrationNumber = new javax.swing.JTextField();
        fieldName = new javax.swing.JTextField();

        setPreferredSize(new java.awt.Dimension(600, 400));

        icon.setIcon(new javax.swing.ImageIcon("/home/timon/work/agrosense/agrosense~core/farm/type/src/main/resources/nl/cloudfarming/client/farm/type/house32.png")); // NOI18N
        icon.setText(org.openide.util.NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.icon.text")); // NOI18N
        icon.setToolTipText(org.openide.util.NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.icon.toolTipText")); // NOI18N

        nameLabel.setText(org.openide.util.NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.nameLabel.text")); // NOI18N
        nameLabel.setToolTipText(org.openide.util.NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.nameLabel.toolTipText")); // NOI18N

        registrationNumberLabel.setText(org.openide.util.NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.registrationNumberLabel.text")); // NOI18N
        registrationNumberLabel.setToolTipText(org.openide.util.NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.registrationNumberLabel.toolTipText")); // NOI18N

        jLabel1.setFont(new java.awt.Font("DejaVu Sans", 0, 24)); // NOI18N
        jLabel1.setText(org.openide.util.NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.jLabel1.text")); // NOI18N

        org.jdesktop.beansbinding.Binding binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, fieldName, org.jdesktop.beansbinding.ELProperty.create("${text}"), jLabel1, org.jdesktop.beansbinding.BeanProperty.create("text"));
        bindingGroup.addBinding(binding);

        fieldRegistrationNumber.setText(org.openide.util.NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.fieldRegistrationNumber.text")); // NOI18N

        fieldName.setText(org.openide.util.NbBundle.getMessage(FarmDetailsPanel.class, "FarmDetailsPanel.fieldName.text")); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(icon)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 515, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(registrationNumberLabel)
                            .addComponent(nameLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 109, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(fieldRegistrationNumber, javax.swing.GroupLayout.PREFERRED_SIZE, 335, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(fieldName, javax.swing.GroupLayout.PREFERRED_SIZE, 335, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap(35, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(37, 37, 37)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(icon, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jLabel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(60, 60, 60)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(nameLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(fieldName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(registrationNumberLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(fieldRegistrationNumber, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(211, Short.MAX_VALUE))
        );

        bindingGroup.bind();
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField fieldName;
    private javax.swing.JTextField fieldRegistrationNumber;
    private javax.swing.JLabel icon;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel nameLabel;
    private javax.swing.JLabel registrationNumberLabel;
    private org.jdesktop.beansbinding.BindingGroup bindingGroup;
    // End of variables declaration//GEN-END:variables

    boolean valid(WizardDescriptor wizardDescriptor) {
        return true;
    }

    void read(WizardDescriptor wizardDescriptor) {
        //TODO implement
    }

    void store(WizardDescriptor wizardDescriptor) {
        //TODO implement
    }

    void validate(WizardDescriptor wizardDescriptor) {
        //TODO implement
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if (!outboundChange) {
            inboundChange = true;
            if (Farm.PROP_NAME.equals(evt.getPropertyName())) {
                fieldName.setText((String) evt.getNewValue());
            }
            if (Farm.PROP_REGISTRATIONNUMBER.equals(evt.getPropertyName())) {
                fieldRegistrationNumber.setText((String) evt.getNewValue());
            }
            inboundChange = false;
        }
    }

    private class NameChangeListener implements DocumentListener {

        @Override
        public void insertUpdate(DocumentEvent e) {
            updateName(e.getDocument());
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            updateName(e.getDocument());
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            updateName(e.getDocument());
        }

        void updateName(Document d) {
            if (!inboundChange) {
                outboundChange = true;
                try {
                    farm.setName(d.getText(0, d.getLength()));
                } catch (BadLocationException ex) {
                    Exceptions.printStackTrace(ex);
                }
                outboundChange = false;
            }
        }
    }

    private class RegistrationChangeListener implements DocumentListener {

        @Override
        public void insertUpdate(DocumentEvent e) {
            updateName(e.getDocument());
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            updateName(e.getDocument());
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            updateName(e.getDocument());
        }

        void updateName(Document d) {
            if (!inboundChange) {
                outboundChange = true;
                try {
                    farm.setRegistrationNumber(d.getText(0, d.getLength()));
                } catch (BadLocationException ex) {
                    Exceptions.printStackTrace(ex);
                }
                outboundChange = false;
            }
        }
    }
}
